// materials.h

#include <windows.h>

#define UNIT 0.1

double circleX( double A, double B, double R, double Y);
int round( double substrate);

class Point
{
public :
	double x, y;
	Point() { }
	Point( double fx, double fy) {
		x = fx;
		y = fy;
	}
};

class DNA
{
public :
	double A, B, R, C1min, C1max, C2min, C2max;
	DNA() { }
	DNA( double fA, double fB, double fR, double fC1max, double fC1min, double fC2max, double fC2min)
	{
		A = (double)fA;
		B = (double)fB;
		R = (double)fR;
		C1max = (double)fC1max;
		C1min = (double)fC1min;
		C2max = (double)fC2max;
		C2min = (double)fC2min;
	}
};

typedef void (*DRAWLINE)( Point p1, Point p2);

class Script
{
public :
	Point P1, P2;
	DRAWLINE pFunc;

	Script() { }
	virtual Point promoterON() = 0;
	virtual double equation( double state) = 0;
	bool phospho_diester_bond();
};

bool Script::phospho_diester_bond()
{
	pFunc( P1, P2);
	return true;
}

class ScriptCircleSuite : public Script
{
public :
	DNA code;


	void aim( DNA atgc, DRAWLINE funcAdress)
	{
		code = atgc;
		pFunc = funcAdress;
	}

	void script( double state)
	{
		if( code.C1min <= state && state <= code.C1max) {
			double x = equation( state);
			if( code.C2min <= x && x <= code.C2max) {
				P2.x = x;
				P2.y = state;
				phospho_diester_bond();
				P1 = P2;
			}
		}
	}

	Point promoterON() {
		double tempX,tempY;
		tempY = code.C1max;
		while( true) {
			if( code.C1min <= tempY && tempY <= code.C1max) {
				tempX = equation( tempY);
				if( code.C2min <= tempX && tempX <= code.C2max) {
					P1.x = tempX;
					P1.y = tempY;
					return P1;
				}
			}
			tempY -= UNIT;
		}
	}

	Point polyadenylation() {
		P2.y = code.C1min;
		P2.x = equation( P2.y);
		phospho_diester_bond();
		return P2;
	}

	double equation( double state)
	{
		return circleX( code.A, code.B, code.R, state);
	}
};

// circle expression
// (x-a)^2 + (y-b)^2 = r~2
double circleX( double a, double b, double r, double y)
{
	// x = Ay^2 + By + C
	double A = 1;
	double B = -2*a;
	double C = pow(y,2) - 2*b*y + pow(a,2) + pow(b,2) - pow(r,2);
	return (-B + sqrt( pow(B,2) - 4*A*C)) / (2*A);
}

int round( double substrate)
{
	int positive = (int)substrate;
	// ؂
	double product = substrate - (double)positive;
	int enzyme = (int)(product*10);
	// TȏȂ
	if( enzyme >= 5)	positive++;
	return positive;
}
